'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.echarts = exports.NO_AXIS = undefined;
exports.initOption = initOption;
exports.createChart = createChart;
exports.updateChart = updateChart;

var _echarts = require('echarts/lib/echarts');

var _echarts2 = _interopRequireDefault(_echarts);

require('echarts/lib/component/tooltip');

require('echarts/lib/component/title');

require('echarts/lib/component/legend');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var NO_AXIS = exports.NO_AXIS = ['gauge', 'graph', 'radar', 'sankey', 'pie', 'treemap', 'wordCloud', 'map', 'heatmap'];

//初始化图表配置参数
function initOption() {
  var type = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'bar';
  var title = arguments[1];
  var subtitle = arguments[2];

  var option = {
    _type: type,
    title: {
      text: title,
      subtext: subtitle
    },
    tooltip: {
      trigger: 'axis'
    },
    legend: {
      data: null
    },
    toolbox: {
      show: false,
      feature: {
        mark: { show: true },
        dataView: { show: true, readOnly: false },
        magicType: { show: true, type: ['line', 'bar'] },
        restore: { show: true },
        saveAsImage: { show: true }
      }
    },
    calculable: true,
    xAxis: NO_AXIS.indexOf(type) <= -1 ? {} : null,
    yAxis: NO_AXIS.indexOf(type) <= -1 ? {} : null,
    series: null
  };

  return option;
}

function _setSeriesType(series, type) {
  series && series.forEach(function (p) {
    if (!p.type) {
      p.type = type;
    }
  });
}

//创建图表
function createChart(render, chartData, option, expandOption, theme) {
  if (chartData) {
    option.series = chartData;
    _setSeriesType(option.series, option._type);
  }

  var chart = _echarts2.default.init(render, theme, expandOption);
  chart.setOption(option);
  return chart;
}

//更新图表
function updateChart(chart, chartData, option) {
  if (chartData) {
    option.series = chartData;
    _setSeriesType(option.series, option._type);
  }

  chart.setOption(option, true);
  chart.resize();
}

exports.echarts = _echarts2.default;