'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getFirstChar = getFirstChar;
exports.getGB2312Pinyin = getGB2312Pinyin;
exports.compare = compare;
exports.compareNumber = compareNumber;
exports.compareDate = compareDate;
exports.compareStringEN = compareStringEN;
exports.compareStringCH = compareStringCH;

var _core = require('../core');

var _core2 = _interopRequireDefault(_core);

var _regexp = require('./regexp');

var regExp = _interopRequireWildcard(_regexp);

var _date = require('./date');

var date = _interopRequireWildcard(_date);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

//取字符串的第一个字符
function getFirstChar(s) {
  if (s === '') {
    return '';
  }

  return (s + '').substr(0, 1);
}

//取得汉字的拼音
function getGB2312Pinyin(str, sp) {
  var i = void 0,
      l = void 0,
      t = void 0,
      p = void 0,
      ret = '';
  if (sp == null) {
    sp = '';
  }

  var _fj$GB2312Pinyin = _core2.default.GB2312Pinyin,
      fonts = _fj$GB2312Pinyin.fonts,
      pinyin = _fj$GB2312Pinyin.pinyin;

  for (i = 0, l = str.length; i < l; i++) {
    if (str.charCodeAt(i) >= 0x4e00) {
      p = fonts.indexOf(str.charAt(i));
      if (p > -1 && p < 3755) {
        for (t = pinyin.length - 1; t > 0; t = t - 2) {
          if (pinyin[t] <= p) {
            break;
          }
        }
        if (t > 0) {
          ret += pinyin[t - 1] + sp;
        }
      }
    }
  }

  return ret.substr(0, ret.length - sp.length);
}

//简单值比较算法
function compare(x, y) {
  var isAsc = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;
  var spC = arguments[3];
  var spC2 = arguments[4];
  var spV = arguments[5];

  spV = spV != null ? spV : -1;
  if (spC && !spC2) {
    //如有禁止排序标记则拍在最低位置
    return -1;
  } else if (!spC && spC2) {
    return 1;
  } else if (spC && spC2) {
    return 0;
  }
  if (x > y) {
    return !isAsc ? -1 : 1;
  } else if (x < y) {
    return !isAsc ? 1 : -1;
  } else {
    return 0;
  }
}

//数字比较算法
function compareNumber(x, y) {
  var isAsc = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;
  var spC = arguments[3];
  var spC2 = arguments[4];
  var spV = arguments[5];

  var r = /[^d|.|-]/g;
  x = (x != null ? x + '' : '').replace(r, '');
  y = (y != null ? y + '' : '').replace(r, '');
  return compare(x * 1, y * 1, isAsc, spC, spC2, spV);
}

//日期比较算法
function compareDate(x, y) {
  var isAsc = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;
  var spC = arguments[3];
  var spC2 = arguments[4];
  var spV = arguments[5];

  var d = '1900-01-01';
  x = date.parse(x == null || x === '' ? d : x);
  y = date.parse(y == null || y === '' ? d : y);
  var z = x - y;

  spV = spV != null ? spV : -1;
  if (spC && !spC2) {
    //如有禁止排序标记则拍在最低位置
    return -1;
  } else if (!spC && spC2) {
    return 1;
  } else if (spC && spC2) {
    return 0;
  }
  return !isAsc ? z * -1 : z;
}

//英文字符串比较算法
function compareStringEN(x, y) {
  var isAsc = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;
  var spC = arguments[3];
  var spC2 = arguments[4];
  var spV = arguments[5];

  x = getFirstChar(x);
  y = getFirstChar(y);
  return compare(x, y, isAsc, spC, spC2, spV);
}

//中文字符串比较算法
function compareStringCH(x, y) {
  var isAsc = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;
  var spC = arguments[3];
  var spC2 = arguments[4];
  var spV = arguments[5];

  if (_core2.default.GB2312Pinyin.fonts) {
    //如果第一个字符非中文的则不获取拼音直接用第一个字符比较
    x = x == null || x === '' ? '' : regExp.chFirst.test(x) ? getGB2312Pinyin(getFirstChar(x)) : getFirstChar(x);
    y = y == null || y === '' ? '' : regExp.chFirst.test(y) ? getGB2312Pinyin(getFirstChar(y)) : getFirstChar(y);
    return compare(x, y, isAsc, spC, spC2, spV);
  } else {
    return compareStringEN(x, y, isAsc, spC, spC2, spV);
  }
}